@@ -39,8 +39,8 @@ flush(stderr()); flush(stdout())
 ### Name: PBmodcomp
 ### Title: Model comparison of mixed models using parametric bootstrap
 ###   methods.
-### Aliases: PBmodcomp PBmodcomp.mer makePBcluster stopPBcluster
-###   plot.XXmodcomp
+### Aliases: PBmodcomp PBmodcomp.lm PBmodcomp.mer getLRT getLRT.lm
+###   getLRT.mer plot.XXmodcomp
 ### Keywords: utilities models inference
 
 ### ** Examples
@@ -54,9 +54,15 @@ PBmodcomp(beet0, beet_no.harv, nsim=20)
 ## Not run: 
 ##D ## Vanilla
 ##D PBmodcomp(beet0, beet_no.harv)
+##D 
 ##D ## Simulate reference distribution separately:
 ##D rr <- PBrefdist(beet0, beet_no.harv, nsim=20)
 ##D PBmodcomp(beet0, beet_no.harv, ref=rr)
+##D 
+##D ## Do computations with multiple processors:
+##D cl <- makeSOCKcluster(rep("localhost", 4))
+##D PBmodcomp(beet0, beet_no.harv, cl=cl)
+##D stopCluster(cl)
 ## End(Not run)
 
 
@@ -70,7 +76,7 @@ flush(stderr()); flush(stdout())
 
 ### Name: PBrefdist
 ### Title: Calculate reference distribution using parametric bootstrap
-### Aliases: PBrefdist PBrefdist.mer
+### Aliases: PBrefdist PBrefdist.mer PBrefdist.lm
 ### Keywords: utilities models
 
 ### ** Examples
@@ -86,7 +92,6 @@ rr
 
 ## Computations can be made in parallel using several processors:
 ## Not run: 
-##D library(snow)
 ##D cl <- makeSOCKcluster(rep("localhost", 4))
 ##D clusterEvalQ(cl, library(lme4))
 ##D clusterSetupSPRNG(cl)
@@ -120,6 +125,26 @@ summary(aov(sugpct~block+sow+harvest+Error(bh), beets))
 
 
 
+cleanEx()
+nameEx("vcovAdj")
+### * vcovAdj
+
+flush(stderr()); flush(stdout())
+
+### Name: vcovAdj
+### Title: Ajusted covariance matrix for linear mixed models according to
+###   Kenward and Roger
+### Aliases: vcovAdj
+### Keywords: function
+
+### ** Examples
+
+(fmLarge <- lmer(Reaction ~ Days + (Days|Subject), sleepstudy))
+## removing Day
+(vcovAdj(fmLarge,detail=0))
+
+
+
 ### * <FOOTER>
 ###
 cat("Time elapsed: ", proc.time() - get("ptime", pos = 'CheckExEnv'),"\n")
